import { h, Host, getElement } from "@stencil/core";
const supportsConstructibleStylesheets = (() => {
    try {
        return !!new CSSStyleSheet();
    }
    catch (e) {
        return false;
    }
})();
/**
 * Dynamically create a constructible stylesheet which is applied to the component.
 * The stylesheet is then cached for future instances of the component.
 * @usage
As a string:
```
@ConstructableStyle() style = `.bg { background: url('assets/${ this.mode }/bg.png'); }`;
```
As a function:
```
@ConstructableStyle() style = () => `.bg { background: url('assets/${ this.mode }/bg.png'); }`;
```
 */
export function ConstructibleStyle(opts = {}) {
    return (target, propertyKey) => {
        if (!opts.cacheKeyProperty) {
            opts.cacheKeyProperty = propertyKey;
        }
        const { componentWillLoad, render } = target;
        if (supportsConstructibleStylesheets) {
            target.componentWillLoad = function () {
                const cssText = (typeof this[propertyKey] === "function" ? this[propertyKey]() : this[propertyKey]);
                const willLoadResult = componentWillLoad && componentWillLoad.call(this);
                const host = getElement(this);
                const root = (host.shadowRoot || host);
                root.adoptedStyleSheets = [...root.adoptedStyleSheets || [], getOrCreateStylesheet(this, target, cssText, opts)];
                return willLoadResult;
            };
        }
        else {
            target.render = function () {
                const cssText = (typeof this[propertyKey] === "function" ? this[propertyKey]() : this[propertyKey]);
                let renderedNode = render.call(this);
                const style = h("style", { type: "text/css" }, cssText);
                if (isHost(renderedNode)) {
                    (getHostChildren(renderedNode) || []).push(style);
                }
                else {
                    renderedNode = h(Host, null, renderedNode);
                }
                return renderedNode;
            };
        }
    };
}
function getOrCreateStylesheet(instance, target, cssText, opts) {
    if (!target.__constructableStylesheets) {
        target.__constructableStylesheets = {};
    }
    let key = instance[opts.cacheKeyProperty];
    if (!target.__constructableStylesheets[key]) {
        target.__constructableStylesheets[key] = new CSSStyleSheet();
        target.__constructableStylesheets[key].replace(cssText);
    }
    return target.__constructableStylesheets[key];
}
function isHost(node) {
    for (let prop in node) {
        if (node.hasOwnProperty(prop)) {
            if (node[prop] === Host) {
                return true;
            }
        }
    }
    return false;
}
function getHostChildren(node) {
    for (let prop in node) {
        if (node.hasOwnProperty(prop)) {
            if (Array.isArray(node[prop])) {
                return node[prop];
            }
        }
    }
}
